@@ -117,6 +117,7 @@ modules = [
     "piczel",
     "pillowfort",
     "pinterest",
+    "pixeldrain",
     "pixiv",
     "pixnet",
     "plurk",
@@ -147,6 +148,7 @@ modules = [
     "tapas",
     "tcbscans",
     "telegraph",
+    "tmohentai",
     "toyhouse",
     "tsumino",
     "tumblr",
